MODULE sbcice_lim
   !!======================================================================
   !!                       ***  MODULE  sbcice  ***
   !! Surface module :  update surface ocean boundary condition over ice
   !!                   covered area using LIM sea-ice model
   !! Sea-Ice model  :  LIM Sea ice model time-stepping
   !!======================================================================
   !! History :  9.0   !  06-06  (G. Madec)  from icestp.F90
   !!----------------------------------------------------------------------
#if defined key_ice_lim
   !!----------------------------------------------------------------------
   !!   'key_ice_lim' :                                   Lim sea-ice model
   !!----------------------------------------------------------------------
   !!----------------------------------------------------------------------
   !!   sbc_ice_lim    : sea-ice model time-stepping and
   !!                    update ocean sbc over ice-covered area
   !!----------------------------------------------------------------------
   USE oce             ! ocean dynamics and tracers
   USE dom_oce         ! ocean space and time domain
   USE ice
   USE iceini
   USE ice_oce         ! ice variables
   USE dom_ice
   USE cpl_oce

   USE sbc_oce         ! Surface boundary condition: ocean fields
   USE sbc_ice         ! Surface boundary condition: ice   fields
   USE sbcblk_core     ! Surface boundary condition: CORE bulk
   USE sbcblk_clio     ! Surface boundary condition: CLIO bulk
   USE albedo

   USE daymod          ! day calendar
   USE phycst          ! Define parameters for the routines
   USE eosbn2          ! equation of state
   USE limdyn
   USE limtrp
   USE limdmp
   USE limthd
   USE limsbc          ! sea surface boundary condition
   USE limdia
   USE limwri
   USE limrst

   USE lbclnk
   USE iom             ! I/O manager library
   USE in_out_manager  ! I/O manager
   USE prtctl          ! Print control

   IMPLICIT NONE
   PRIVATE

   PUBLIC sbc_ice_lim ! routine called by step.F90
   
   !! * Substitutions
#  include "domzgr_substitute.h90"
#  include "vectopt_loop_substitute.h90"
   !!----------------------------------------------------------------------
   !!   OPA 9.0 , LOCEAN-IPSL (2006) 
   !! $Header: /
   !! Software governed by the CeCILL licence (modipsl/doc/NEMO_CeCILL.txt)
   !!----------------------------------------------------------------------

CONTAINS

   SUBROUTINE sbc_ice_lim( kt, kblk )
      !!---------------------------------------------------------------------
      !!                  ***  ROUTINE ice_stp  ***
      !!                   
      !! ** Purpose :   update the ocean surface boundary condition via the 
      !!                Louvain la Neuve Sea Ice Model time stepping 
      !!
      !! ** Method  :   ice model time stepping
      !!              - call the ice dynamics routine 
      !!              - call the ice advection/diffusion routine 
      !!              - call the ice thermodynamics routine 
      !!              - call the routine that computes mass and 
      !!                heat fluxes at the ice/ocean interface
      !!              - save the outputs 
      !!              - save the outputs for restart when necessary
      !!
      !! ** Action  : - time evolution of the LIM sea-ice model
      !!              - update all sbc variables below sea-ice:
      !!                utau, vtau, qns , qsr, emp , emps
      !!---------------------------------------------------------------------
      INTEGER, INTENT(in) ::   kt      ! ocean time step
      INTEGER, INTENT(in) ::   kblk    ! type of bulk (=3 CLIO, =4 CORE)
      !!
      INTEGER  ::   ji, jj   ! dummy loop indices
      REAL(wp), DIMENSION(jpi,jpj) ::   alb_oce_os   ! albedo of the ocean under overcast sky
      REAL(wp), DIMENSION(jpi,jpj) ::   alb_oce_cs   ! albedo of the ocean under clear sky
      REAL(wp), DIMENSION(jpi,jpj) ::   alb_ice_os   ! albedo of the ice under overcast sky
      REAL(wp), DIMENSION(jpi,jpj) ::   alb_ice_cs   ! albedo of ice under clear sky
      !!----------------------------------------------------------------------

      IF( kt == nit000 ) THEN
         IF(lwp) WRITE(numout,*)
         IF(lwp) WRITE(numout,*) 'sbc_ice_lim : update ocean surface boudary condition' 
         IF(lwp) WRITE(numout,*) '~~~~~~~~~~~   via Louvain la Neuve Ice Model (LIM) time stepping'

         CALL ice_init

      ENDIF

      IF( MOD( kt-1, nn_fsbc ) == 0 ) THEN
         !
         ! ... mean surface ocean current at I-point (F-point with sea-ice indexation)
         DO jj = 2, jpj
            DO ji = fs_2, jpi   ! vector opt.
               ui_oce(ji,jj) = 0.5 * ( ssu_m(ji-1,jj  ) + ssu_m(ji-1,jj-1) )
               vi_oce(ji,jj) = 0.5 * ( ssv_m(ji  ,jj-1) + ssv_m(ji-1,jj-1) )
            END DO
         END DO
         CALL lbc_lnk( ui_oce, 'I', -1. )   ! I-point (i.e. F-point with ice indices)
         CALL lbc_lnk( vi_oce, 'I', -1. )   ! I-point (i.e. F-point with ice indices)

         ! ... masked sea surface freezing temperature [Kelvin] (set to rt0 over land)
         tfu(:,:) = tfreez( sss_m ) +  rt0 

         ! ... ice and ocean albedo
         CALL blk_albedo( alb_ice_os , alb_oce_os , alb_ice_cs , alb_oce_cs )


         ! ... surface ice boundary condition (stress at I-point, latente, sensible, etc....
         !!gm preciser la list exacte
         SELECT CASE( kblk )
         CASE( 3 )           ! CLIO bulk formulation
            CALL blk_ice_clio( sist     , ui_ice   , vi_ice   , alb_ice_cs, alb_ice_os,            &
               &                                     utaui_ice, vtaui_ice , qns_ice   , qsr_ice,   &
               &                                     qla_ice  , dqns_ice  , dqla_ice  ,            &
               &                                     tprecip  , sprecip   ,                        &
               &                                     fr1_i0   , fr2_i0    , 'B'  )
         CASE( 4 )           ! CORE bulk formulation
            CALL blk_ice_core( sist     , ui_ice   , vi_ice   , alb_ice_cs,                      &
               &                                     utaui_ice, vtaui_ice , qns_ice , qsr_ice,   &
               &                                     qla_ice  , dqns_ice  , dqla_ice,            &
               &                                     tprecip  , sprecip   ,                      &
               &                                     fr1_i0   , fr2_i0  )
         END SELECT

         IF(ln_ctl) THEN         ! print mean trends (used for debugging)
            CALL prt_ctl_info( 'Ice Forcings ' )
            CALL prt_ctl( tab2d_1=tprecip  ,clinfo1=' precip   : ', tab2d_2=sprecip  , clinfo2=' Snow     : ' )
            CALL prt_ctl( tab2d_1=utaui_ice,clinfo1=' utaui_ice: ', tab2d_2=vtaui_ice, clinfo2=' vtaui_ice: ' )
            CALL prt_ctl( tab2d_1=hsnif    ,clinfo1=' hsnif  1 : ', tab2d_2=hicif    , clinfo2=' hicif    : ' )
            CALL prt_ctl( tab2d_1=frld     ,clinfo1=' frld   1 : ', tab2d_2=sist     , clinfo2=' sist     : ' )
         ENDIF

         ! ---------------- !
         !  Ice model step  !
         ! ---------------- !
         ;                              CALL lim_rst_opn  ( kt )      ! Open Ice restart file
         ;                              CALL lim_dyn      ( kt )      ! Ice dynamics    ( rheology/dynamics )
         ;                              CALL lim_trp      ( kt )      ! Ice transport   ( Advection/diffusion )
         IF( ln_limdmp )                CALL lim_dmp      ( kt )      ! Ice damping 
         ;                              CALL lim_thd      ( kt )      ! Ice thermodynamics 
         ;                              CALL lim_sbc      ( kt )      ! Ice/Ocean Mass & Heat fluxes 
         IF( MOD( kt+nn_fsbc-1, ninfo ) == 0 .OR.   &
            &  ntmoy == 1 )             CALL lim_dia      ( kt )      ! Ice Diagnostics 
         ;                              CALL lim_wri      ( kt )      ! Ice outputs 
         IF( lrst_ice )                 CALL lim_rst_write( kt )      ! Ice restart file 
         !
      ENDIF
      !
   END SUBROUTINE sbc_ice_lim

#else
   !!----------------------------------------------------------------------
   !!   Default option           Dummy module          NO LIM sea-ice model
   !!----------------------------------------------------------------------
CONTAINS
   SUBROUTINE sbc_ice_lim ( kt, kblk )     ! Dummy routine
      WRITE(*,*) 'sbc_ice_lim: You should not have seen this print! error?', kt, kblk
   END SUBROUTINE sbc_ice_lim
#endif

   !!======================================================================
END MODULE sbcice_lim
